package com.xy.config;
 
import lombok.Data;
import me.chanjar.weixin.common.api.WxConsts;
import me.chanjar.weixin.mp.api.WxMpMessageHandler;
import me.chanjar.weixin.mp.api.WxMpMessageRouter;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.api.impl.WxMpServiceImpl;
import me.chanjar.weixin.mp.config.impl.WxMpDefaultConfigImpl;
import me.chanjar.weixin.mp.config.impl.WxMpRedisConfigImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.Map;
import java.util.stream.Collectors;


@Component
@Data
public class WeChatOfficialConfig {
    public static final String ENCRYPT_TYPE = "encrypt_type";
    public static final String RAW = "raw";
    public static final String AES = "aes";
    public static final String SIGNATURE = "signature";
    public static final String ECHO_STR = "echostr";
    public static final String NONCE = "nonce";
    public static final String TIMESTAMP = "timestamp";


    private static final String APP_ID = "wx95d729fe985ef9de";
    private static final String SECRET = "e76dffd8329ad6c5943cc5e1f6ff95f2";
    private static final String TOKEN = "111";
    private static final String AES_KEY = "111";
    public static final String MSG_SIGNATURE = "msg_signature";
    public static final String SCAN_HANDLER = "scanHandler";
    public static final String SUBSCRIBE_HANDLER = "subscribeHandler";
    public static final String UNSUBSCRIBE_HANDLER = "unSubscribeHandler";

    /**
     * 微信服务号 登录 相关对象
     */
    private WxMpService officialAccountLoginService;
    private WxMpDefaultConfigImpl config;
    private WxMpMessageRouter wxMpMessageRouter;

    @Autowired
    private Map<String, WxMpMessageHandler> wxMpMessageHandlers;


    @PostConstruct
    private void init() {
        officialAccountLoginService = getWxMpService();
        initConfig();
        initRouter();
    }

    /**
     * 初始化微信服务号
     *
     * @return {@link WxMpService}
     */
    public WxMpService getWxMpService() {
        WxMpService service = new WxMpServiceImpl();
        WxMpDefaultConfigImpl configStorage = new WxMpDefaultConfigImpl();
        configStorage.setAppId(APP_ID);
        configStorage.setSecret(SECRET);
        configStorage.setToken(TOKEN);
        configStorage.setAesKey(AES_KEY);
        service.setWxMpConfigStorage(configStorage);
        return service;
    }

    /**
     * 初始化路由器
     * 扫码、关注、取消关注
     */
    private void initRouter() {
        wxMpMessageRouter = new WxMpMessageRouter(officialAccountLoginService);
        wxMpMessageRouter.rule().async(false).msgType(WxConsts.XmlMsgType.EVENT).event(WxConsts.EventType.SUBSCRIBE).handler(wxMpMessageHandlers.get(SUBSCRIBE_HANDLER)).end();
        wxMpMessageRouter.rule().async(false).msgType(WxConsts.XmlMsgType.EVENT).event(WxConsts.EventType.SCAN).handler(wxMpMessageHandlers.get(SCAN_HANDLER)).end();
        wxMpMessageRouter.rule().async(false).msgType(WxConsts.XmlMsgType.EVENT).event(WxConsts.EventType.UNSUBSCRIBE).handler(wxMpMessageHandlers.get(UNSUBSCRIBE_HANDLER)).end();
    }

    /**
     * 初始化配置信息
     */
    private void initConfig() {
        config = new WxMpDefaultConfigImpl();
        config.setAppId(APP_ID);
        config.setToken(TOKEN);
        config.setSecret(SECRET);
    }

}
